# sactive-web

A dependency injection web framework for Node.js.

[![Build status][travis-image]][travis-url]
[![Coverage][cov-image]][cov-url]
[![NPM version][npm-image]][npm-url]
[![NPM Download][npm-download]][npm-url]
[![License][license-image]][license-url]

[![NPM](https://nodei.co/npm/sactive-web.png?downloads=true)](https://nodei.co/npm/sactive-web/)

## Installation
```bash
npm install sactive-web
```

## Features

- Dependency injection.
- Routing, based on [koa-router](https://github.com/alexmingoia/koa-router).
- Support async Function, common function.
- Parameter validate and normalize.
- Query validate and normalize.
- FormData validate and normalize.
- Response transform.
- Support pug template engine, based on [koa-views](https://github.com/queckezz/koa-views).
- Based on [Koa](https://github.com/koajs/koa).

> async Function require node 7.6+.

## Example

```javascript
const SactiveWeb = require('sactive-web');

let demo = {
  name: 'hello',
  method: 'get',
  path: '/demo/hello',
  handler: function(ctx, next) {
    ctx.body = 'Hello SActive !!!';
  }
};

let app = new SactiveWeb();
app.route(demo);

app.init();
app.listen(8080);
```

## Documentation
- Refer to the [Document web site](https://www.shipengqi.top/sactive-web).

> My English is poor, so my documents are all Chinese.

## Babel setup
If you're not using node `v7.6+`, you can use `babel`:

```javascript
//entry file
require('babel-register');
const app = require('./app');
```

Add the following to `.babelrc`:
```javascript
{
  "presets": [
    ["env", {
      "targets": {
        "node": true
      }
    }]
  ]
}
```


## Examples
```bash
git clone git@github.com:shipengqi/sactive-web.git
cd ./sactive-web
npm install
cd ./example
```

## Tests
Install the dependencies, then run `npm test`:
``` bash
npm install
npm test

#coverage
npm run test:cov
```

## TODO
- Engilsh Documentation

[npm-image]: https://img.shields.io/npm/v/sactive-web.svg
[npm-url]: https://www.npmjs.com/package/sactive-web
[travis-image]: https://travis-ci.org/shipengqi/sactive-web.svg?branch=master
[travis-url]: https://www.travis-ci.org/shipengqi/sactive-web
[cov-image]: https://codecov.io/gh/shipengqi/sactive-web/branch/master/graph/badge.svg
[cov-url]: https://codecov.io/gh/shipengqi/sactive-web
[npm-download]: https://img.shields.io/npm/dw/sactive-web.svg
[license-image]: http://img.shields.io/npm/l/sactive-web.svg
[license-url]: ./LICENSE
